%%%%%%%%%%%%%%
% TAR_UR_C.M %
%%%%%%%%%%%%%%
% This Matlab procedure estimates a constrained TAR(p+1).
% It tests for a threshold and for unit roots.

% written by:

% Bruce E. Hansen
% Department of Economics
% Social Science Building
% University of Wisconsin
% Madison, WI 53706-1393
% bhansen@ssc.wisc.edu
% http://www.ssc.wisc.edu/~bhansen/

% This procedure implements the estimation and testing methods described in
% "Threshold Autoregression with a Near Unit Root" by Mehmet Caner and Bruce E. Hansen.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Format:

% call tar_ur_c(y,p,mmin,mmax,m);

% Inputs:

% y        	vector of data
% p           	# of AR lags, must be greater or equal to one
% mmin        	Minimal Delay Order for testing
% mmax		Maximal Delay Order for testing
% m		Delay Order (fixed) for estimation
%		Set to 0 for m to be estimated by minimizing SSE



% Global Controls:

% thresh_		Type of threshold variable z
% 		1 for long difference, z(t-1)=y(t-1)-y(t-(m-1))
% 		2 for lagged difference, z(t-1)=y(t-m)-y(t-m-1)
% 		3 for lagged level, z(t-1)=y(t-m)
% trim_		Trimming Range (normalized threshold, lambda)
% 		1 for [.15,.85]
% 		2 for [.10,.90]
% 		3 for [.05,.95]
% trend_		1 to include linear time trend
% 		0 to exclude linear time trend
% graph_       		set to 1 to display graphs, else 0
% xdate_       		nx1 vector, equal in length to dependent variable, for displaying predicted values in graph
% 		For default of undated x-axis, set _xdate=0;
% boot_     	# of bootstrap replications (1 or greater)
% adj_		Adjustment to Gaussian likelihood for Jacobian transformations
% coef_		AR lag indices that switch between regimes
% 		Set to zero for no AR lags to switch 
% 		(Intercept and y(t-1) switch by default)
		

% Default Global Settings %
function tar_ur_c(dat,p,mmin,mmax,m);
global thresh_;thresh_=1;
global trim_;trim_=1;
global graph_;
global trend_;trend_ = 0;
global boot_;boot_ = 10000; % Change the number of bootraps here %
global adj_;adj_=0;
global xdate_;
global test_;test_=0;
global coef_;coef_=[1 2] ;

%%%%%%%%%%%%%%%%%% 
% Main Procedure %
%%%%%%%%%%%%%%%%%%

t=length(dat(:,1));
n=t-1-p;
mn=mmax-mmin+1;
ms=(mmin:mmax)';
if sum(coef_)==0
    xi_ns=(1:p)'+2+trend_;
else
    temp=zeros(1,p);
    for i=1:length(coef_)
        temp=temp+((1:p)==coef_(i));
    end;
    ind=0;
    xi_ns=0;
    for i=1:p
        if temp(i)==0
            if ind==0
                xi_ns=i;
                ind=1;
            else
                xi_ns=[xi_ns;i];
            end;
        end;
    end;
    xi_ns=xi_ns+2+trend_;    
end;
if trend_==0
    if sum(coef_)==0
        xi_s=[1;2];
    else
        xi_s=[1;2;coef_'+2];
    end;
elseif trend_==1
    if sum(coef_)==0
        xi_s=[1;2;3];
    else
        xi_s=[1;2;3;coef_'+3];
    end;
end;
% Linear Model %
dy=dat(2:t)-dat(1:t-1);
y=dy(1+p:t-1);
x=ones(n,1);
 if trend_==1
     x=[x,(1:n)'];
 end;
 
 x=[x,dat(1+p:t-1),dy(p:t-2)];
 k=2;
 while k<=p
     x=[x,dy(p+1-k:t-1-k)];
     k=k+1;
 end;
 
 kx=length(x(1,:));
 kc=length(coef_)+2+trend_;
 xx=inv(x'*x);
 bols=xx*(x'*y);
 eols=y-x*bols;
 sols=eols'*eols;
 sigols=sols/(n-kx);
 seols=sqrt(diag(xx)*sigols);
 rho=bols(2+trend_);
 tadf=rho/seols(2+trend_);
 ar0=bols(3+trend_:2+trend_+p);
 
%%%%%%%%%%%%%%%%% Threshold Model Estimation %%%%%%%%%%%%%%%%%%%%

[mhat,lam,ee,bs_s,bs_ns,d1,d2,ws,w,r1,r2,t1,t2,ts]=tur_est(dat,p,mmin,mmax,m);

%%%%%% Graphs of Regime Classification %%%%%%%%%%
if graph_==1;
    if xdate_==0
        xdate=(1:n)';
    else
        xdate=xdate_;
    end;
    yp=dat(p+2:t);
    % missing values %
    temp_yp1=yp;
    temp_yp2=yp;
    for i=1:length(d1)
        if d1(i)==1
            temp_yp1(i)=nan;
        end;
        if d2(i)==1
            temp_yp2(i)=nan;
        end;
    end;
    yy=[temp_yp2,temp_yp1];
    figure;
    plot(xdate,yy(:,1),'+',xdate,yy(:,2),'o');
    legend('Regime 1','Regime 2');
    grid on;
end;


% Bootstrap, Unconstrained Estimates %
y0=dat-mean(dat)';
y0=y0(1:1+p);
aa=zeros(1+p,1);
aa(1)=1+rho;
aa(1:p)=aa(1:p)+ar0;
aa(2:1+p)=aa(2:1+p)-ar0;
bootw = zeros(boot_,1);
bootws=zeros(boot_,mn);
bootts=zeros(boot_,kc);
ib=1;
while ib<=boot_
    % Recserar %
    eols_c=eols(ceil(unifrnd(0,1,t,1)*n));
    datb=y0;
    for i=(length(y0(:,1))+1):(length(eols_c(:,1)))
        datbb=eols_c(i,:);
        for j=1:length(y0(:,1))
            datbb=datbb+aa(j,:).*datb(i-j,:);
        end;
        datb=[datb;datbb];
    end;
    [mb,lb,eb,bb1,bb2,d1b,d2b,wsb,wb,r1b,r2b,t1b,t2b,tsb]=tur_est(datb,p,mmin,mmax,m);
    bootw(ib)=wb;
    bootws(ib,:)=wsb';
    bootts(ib,:)=tsb';
    ib=ib+1;
end;
pw=mean(bootw>w);
pws=mean(bootws>(ws*ones(1,length(bootws(:,1))))')';
pts=mean(bootts>(ts*ones(1,length(bootts(:,1))))')';

k=1;
while k<=mn
    bootws(:,k)=sort(bootws(:,k),1);
    k=k+1;
end;

bootw=sortrows(bootw,1);
crlevel=[.9 .95 .99];
crws=bootws(round(boot_*crlevel),:)';
crw=bootw(round(boot_*crlevel))';

% Bootstrap, Constrained Estimates %

aa=zeros(1+p,1);
aa(1)=1;
aa(1:p)=aa(1:p)+ar0;
aa(2:1+p)=aa(2:1+p)-ar0;
bootw = zeros(boot_,1);
bootws = zeros(boot_,mn);
bootr1 = zeros(boot_,mn);
bootr2 = zeros(boot_,mn);
boott1 = zeros(boot_,mn);
boott2 = zeros(boot_,mn);
ib=1;
while ib<=boot_
    % Recserar %
    eols_c=eols(ceil(unifrnd(0,1,t,1)*n));
    datb=y0;
    for i=(length(y0(:,1))+1):(length(eols_c(:,1)))
        datbb=eols_c(i,:);
        for j=1:length(y0(:,1))
            datbb=datbb+aa(j,:).*datb(i-j,:);
        end;
        datb=[datb;datbb];
    end;
    [mb,lb,eb,bb1,bb2,d1b,d2b,wsb,wb,r1b,r2b,t1b,t2b,tsb]=tur_est(datb,p,mmin,mmax,m);
    bootw(ib)=wb;
    bootws(ib,:)=wsb';
    bootr1(ib,:)=r1b';
    bootr2(ib,:)=r2b';
    boott1(ib,:)=t1b';
    boott2(ib,:)=t2b';
    ib=ib+1;
end;
% Compare vectors %
pw_ur=mean(bootw>w)';
pws_ur=mean(bootws>(ws*ones(1,length(bootws(:,1))))')';
pr1=mean(bootr1>(r1*ones(1,length(bootr1(:,1))))')';
pr2=mean(bootr2>(r2*ones(1,length(bootr2(:,1))))')';
pt1=mean(boott1>(t1*ones(1,length(boott1(:,1))))')';
pt2=mean(boott2>(t2*ones(1,length(boott2(:,1))))')';
k=1;
while k<=mn;
    bootr1(:,k)=sort(bootr1(:,k));
    bootr2(:,k)=sort(bootr2(:,k));
    boott1(:,k)=sort(boott1(:,k));
    boott2(:,k)=sort(boott2(:,k));
    k=k+1;
end;
crr1=bootr1(round(boot_*crlevel),:)';
crr2=bootr2(round(boot_*crlevel),:)';
crt1=boott1(round(boot_*crlevel),:)';
crt2=boott2(round(boot_*crlevel),:)';

if trend_==0
    if trim_==1
        acrr1=[10.84,12.75,16.97];
        acrr2=[11.31,13.24,17.50];
        acrt=[2.97,3.26,3.82];
        apr1=1-chi2cdf(1.11284+1.13011*r1,8);
        apr2=1-chi2cdf(-0.0107136+1.06417*r2,7);
        apt1=1-chi2cdf(1.47612-0.0230281*t1+1.04816*(t1.^2),6);
        apt2=1-chi2cdf(1.47612-0.0230281*t2+1.04816*(t2.^2),6);
    elseif trim_==2 
        acrr1=[10.84,12.75,16.97];
        acrr2=[11.31,13.24,17.50];
        acrt=[2.97,3.26,3.82];
        apr1=1-chi2cdf(1.11284+1.13011*r1,8);
        apr2=1-chi2cdf(-0.0107136+1.06417*r2,7);
        apt1=1-chi2cdf(1.47612-0.0230281*t1+1.04816*(t1.^2),6);
        apt2=1-chi2cdf(1.47612-0.0230281*t2+1.04816*(t2.^2),6);
    elseif trim_==3
        acrr1=[11.35,13.29,17.51];
        acrr2=[12.04,14.03,18.24];
        acrt=[3.05,3.34,3.89]; 
        apr1=1-chi2cdf(0.783900+1.10665*r1,8);
        apr2=1-chi2cdf(-0.572447+1.04393*r2,7);
        apt1=1-chi2cdf(2.57297+1.63643*t1+1.03960*(t1.^2),11);
        apt2=1-chi2cdf(2.57297+1.63643*t2+1.03960*(t2.^2),11);
    end;
elseif trend_==1;
    if trim_==1
        acrr1=[14.06,16.24,20.75];
        acrr2=[14.32,16.49,21.00];
        acrt=[3.40,3.69,4.26];
        apr1=1-chi2cdf(0.455551+1.10432*r1,10);
        apr2=1-chi2cdf(-0.284809+1.04342*r2,9);
        apt1=1-chi2cdf(6.47943+3.82216*t1+0.974897*(t1.^2),22);
        apt2=1-chi2cdf(6.47943+3.82216*t2+0.974897*(t2.^2),22);
    elseif trim_==2
        acrr1=[14.30,16.44,20.98];
        acrr2=[14.66,16.81,21.36];
        acrt=[3.45,3.73,4.29];
        apr1=1-chi2cdf(0.281771+1.09842*r1,10);
        apr2=1-chi2cdf(-0.0204306+1.09231*r2,10);
        apt1=1-chi2cdf(5.93011+3.74162*t1+1.00551*(t1.^2),22);
        apt2=1-chi2cdf(5.93011+3.74162*t2+1.00551*(t2.^2),22);
    elseif trim_==3
        acrr1=[14.56,16.73,21.21];
        acrr2=[15.07,17.22,21.73];
        acrt=[3.49,3.77,4.32];
        apr1=1-chi2cdf(0.102428+1.0908*r1,10);
        apr2=1-chi2cdf(-0.349898+1.08470*r2,10);
        apt1=1-chi2cdf(4.96307+3.96004*t1+0.985536*(t1.^2),22);
        apt2=1-chi2cdf(4.96307+3.96004*t2+0.985536*(t2.^2),22);
    end;
end;

% Print Output to Screen %
nn=[sum(d1)';sum(d2)'];
xname='Intercpt';
if trend_==1
    xname=[xname;'Trend   '];
end;
xname=[xname;'Y(t-1)  ';'DY(t-1) '];
k=2;
while k<=p
    if k<10
        kname=strcat('DY(t-0',num2str(k),')');
    end;
    if k>=10
        kname=strcat('DY(t-',num2str(k),')');
    end;
    xname=[xname;kname];
    k=k+1;
end;
hname=['Variable     ','Estimate     ','St Error'];
disp(' ');
fprintf('Number of Observations:       %u\n',n);
fprintf('Lags of DY:                   %u\n',p);
fprintf('Bootstrap Replications:       %u\n',boot_);

if thresh_==1;
    disp('Threshold is Long Difference');
end;
if thresh_==2;
    disp('Threshold is Lagged Difference');
end;
if thresh_==3;
    disp('Threshold is Lagged Level');
end;

if trim_==1;
    disp('Lambda Trimming Region: [.15,.85]');
end;
if trim_==2;
    disp('Lambda Trimming Region: [.10,.90]');
end;
if trim_==3;
    disp('Lambda Trimming Region: [.05,.95]');
end;

disp(' ');
disp(' ');

disp('Linear Model: ');
disp('------------');
fprintf('Residual Variance:            %f\n',sigols);
fprintf('ADF Statistic:                %f\n',tadf);
disp('');
disp('');
fprintf('%s\n',hname);
disp('----------------------------------');
for i=1:length(bols)
    fprintf('%s     %f    %f\n',xname(i,:),bols(i),seols(i));
end;
disp('----------------------------------');
disp(' ');

disp('Threshold Model:');
disp('----------------');
if m==0
    fprintf('Estimated Delay Parameter:    %f\n',mhat);
    disp('(Minimizes sum-of-squared errors)');
else
    fprintf('Delay Order:                  %f\n',m);
end;
fprintf('Residual Variance:            %f\n',ee/(n-kx*2));
fprintf('Gaussian Log-Likelihood:      %f\n',-(n/2)*(1+log(2*3.14159*ee/n)));
fprintf('Threshold Estimate:           %f\n',lam);
fprintf('Observations in Regime 1:     %u   %u\n',nn(1),nn(1)/n);
fprintf('Observations in Regime 2:     %u   %u\n',nn(2),nn(2)/n);
disp(' ');
disp(' ');
disp('                Regime 1 Estimates               Regime 2 Estimates');
disp('              ------------------------           ----------------------');
disp('Variable      Estimate       St Error           Estimate      St Error');
for i=1:length(bs_s(:,1))
    fprintf('%s       %f      %f            %f     %f\n',xname(xi_s(i),:),bs_s(i,1),bs_s(i,2),bs_s(i,3),bs_s(i,4));
end;
disp('');
disp('Non-Switching Coefficients');
fprintf('%s\n',hname);
disp('----------------------------------');
for i=1:length(bs_ns)
    fprintf('%s     %f    %f\n',xname(xi_ns(i),:),bs_ns(i,1),bs_ns(i,2));
end;
disp(' ');
disp(' ');
disp('-------------------------------------------------------------------------');
disp('Wald Tests for Threshold Effect, Fixed Delay Parameter');
disp('Wald Tests for Threshold Effect, Fixed Delay Parameter');
disp('     Delay      Wald        Boot.       Critical   Values        Stat.       UnitRoot');
disp('     Order      Stat        10%         5%         1%            P-Value     P-Value');
for i=1:length(ms)
    fprintf('     %f   %f   %f   %f   %f   %f   %f\n',ms(i),ws(i),crws(i,1),crws(i,2),crws(i,3),pws(i),pws_ur(i))
end;
if mn>1
    disp('Wald Test for Threshold Effect, Estimated Delay Parameter');
    disp('     Delay      Wald        Boot.       Critical   Values        Stat.       UnitRoot');
    disp('     Order      Stat        10%         5%         1%            P-Value     P-Value');
    fprintf('     %f   %f   %f   %f   %f   %f   %f\n',mhat,w,crw(1),crw(2),crw(3),pw,pw_ur);
    disp(' ');
    disp(' ');
end;
disp('-------------------------------------------------------------------------');
disp('Wald Tests for Threshold Effect in Individual Coefficients');
disp('Variable     Wald     Bootstr');
disp('             Stat     P-Value');
for i=1:length(ts(:,1))
    fprintf('%s   %f   %f\n',xname(xi_s(i),:),ts(i),pts(i));
end;
disp(' ');
disp(' ');
disp('-------------------------------------------------------------------------');
disp(' ');
disp(' ');
disp('2-Sided Wald Test for Unit Root (R_2T)');
disp(' ');
disp('Asymptotic Critical Values');
disp('--------------------------');
disp('10%         5%        1%');
fprintf('%f  %f  %f\n',acrr2(1),acrr2(2),acrr2(3));
disp(' ');

disp('     Delay      Wald        Boot.       Critical   Values      Boot.       Asym.');
disp('     Order      Stat        10%         5%         1%          P-Value     P-Value');
for i=1:length(ms)
    fprintf('     %f   %f   %f   %f   %f   %f   %f\n',ms(i),r2(i),crr2(i,1),crr2(i,2),crr2(i,3),pr2(i),apr2(i));
end;
disp(' ');
disp(' ');

disp('1-Sided Wald Test for Unit Root (R_1T)');
disp(' ');
disp('Asymptotic Critical Values');
disp('--------------------------');
disp('10%         5%        1%');
fprintf('%f  %f  %f\n',acrr1(1),acrr1(2),acrr1(3));
disp('     Delay      Wald        Boot.       Critical   Values      Boot.       Asym.');
disp('     Order      Stat        10%         5%         1%          P-Value     P-Value');
for i=1:length(ms)
    fprintf('     %f   %f   %f   %f   %f   %f   %f\n',ms(i),r1(i),crr1(i,1),crr1(i,2),crr1(i,3),pr1(i),apr1(i));
end;
disp(' ');
disp(' ');
disp('t Tests for Stationarity');
disp(' ');
disp('Asymptotic Critical Values');
disp('--------------------------');
disp('10%         5%        1%');
fprintf('%f  %f  %f\n',acrt(1),acrt(2),acrt(3));
disp(' ');

disp('t_1 Test');
disp(' ');
disp('     Delay      t           Boot.       Critical   Values      Boot.       Asym.');
disp('     Order      Stat        10%         5%         1%          P-Value     P-Value');
for i=1:length(ms)
    fprintf('     %f   %f   %f   %f   %f   %f   %f\n',ms(i),t1(i),crt1(i,1),crt1(i,2),crt1(i,3),pt1(i),apt1(i));
end;
disp(' ');

disp('t_2 Test');
disp(' ');
disp('     Delay      t           Boot.       Critical   Values      Boot.       Asym.');
disp('     Order      Stat        10%         5%         1%          P-Value     P-Value');
for i=1:length(ms)
    fprintf('     %f   %f   %f   %f   %f   %f   %f\n',ms(i),t2(i),crt2(i,1),crt2(i,2),crt2(i,3),pt2(i),apt2(i));
end;
disp(' ');
        
        

%%%%%%%%%%%%%%%%%%%%%%%%%
 % Computation Procedure %
 %%%%%%%%%%%%%%%%%%%%%%%%%

function [mhat,lam,ee,bs_s,bs_ns,d1,d2,ws,w,r1,r2,t1,t2,ts]=tur_est(dat,p,mmin,mmax,m);
global trend_;
global thresh_;
global test_;
global trim_;
global coef_;

t=length(dat(:,1));
n=t-1-p;
mn=mmax-mmin+1;
ms=mmin:1:mmax;
dy=dat(2:t)-dat(1:t-1);
y=dy(1+p:t-1);
x=ones(n,1);
if trend_==1
    x=[x,(1:n)'];
end;
x=[x,dat(1+p:t-1),dy(p:t-2)];
ki=2;
while ki<=p
    x=[x,dy(p+1-ki:t-1-ki)];
    ki=ki+1;
end;
if sum(coef_)==0
    xi_ns=(1:p)'+2+trend_;
else
    temp=zeros(1,p);
    for i=1:length(coef_)
        temp=temp+((1:p)==coef_(i));
    end;
    ind=0;
    xi_ns=0;
    for i=1:p
        if temp(i)==0
            if ind==0
                xi_ns=i;
                ind=1;
            else
                xi_ns=[xi_ns;i];
            end;
        end;
    end;
    xi_ns=xi_ns+2+trend_;    
end;
if trend_==0
    if sum(coef_)==0
        xi_s=[1;2];
    else
        xi_s=[1;2;coef_'+2];
    end;
elseif trend_==1
    if sum(coef_)==0
        xi_s=[1;2;3];
    else
        xi_s=[1;2;3;coef_'+3];
    end;
end;
xs=x(:,xi_s);
xns=x(:,xi_ns);
if thresh_==1
    qs=dat(1+p:t-1)-dat(1+p-mmin:t-1-mmin);
    mi=mmin+1;
    while mi<=mmax
        qs=[qs,dat(1+p:t-1)-dat(1+p-mi:t-1-mi)];
        mi=mi+1;
    end;
elseif thresh_==2;
    qs=dat(1+p-mmin+1:t-1-mmin+1)-dat(1+p-mmin:t-1-mmin)
    mi=mmin+1;
    while mi<=mmax
        qs=[qs,dat(1+p-mi+1:t-1-mi+1)-dat(1+p-mi:t-1-mi)];
        mi=mi+1;
    end;
elseif thresh_==3;
    qs=dat(1+p-mmin+1:t-mmin);
    mi=mmin+1;
    while mi<=mmax;
        qs=[qs,dat(1+p-mi+1:t-mi)];
        mi=mi+1;
    end;
end;
kx=length(x(1,:));
ks=length(coef_)+2+trend_;
xx=inv(x'*x);
bols=xx*(x'*y);
eols=y-x*bols;
sols=eols'*eols;
smins=zeros(mn,1);
lams=zeros(mn,1);
ws = zeros(mn,1);
r1 = zeros(mn,1);
r2 = zeros(mn,1);
t1 = zeros(mn,1);
t2 = zeros(mn,1);
xx = x*xx;
rur=[2;(ks+2)]+trend_;
pi1=.2-.05*trim_;
pi2=.8+.05*trim_;
pi_n=round((pi2-pi1)*100)+1;
mi=1;
while mi<=mn;
    q=qs(:,mi);
    qq=unique(q);
    rq=length(qq(:,1));
    if rq>100;
        qq=qq(round((pi1:.01:pi1+((pi_n-1)*.01))*rq));
    else
        qq=qq(floor(n*pi1):ceil(n*pi2));
    end;
    qn=length(qq(:,1));
    s=zeros(qn,1);
    qi=1;
    while qi<=qn
        d1=xs.*((q<qq(qi))*ones(1,length(xs(1,:))));
        d1=d1-x*(xx'*d1);
        e=eols-d1*(eols'/d1')';
        s(qi)=e'*e;
        qi=qi+1;
    end;
    [temp,qi]=min(s);
    lam=qq(qi);
    smin=s(qi);
    ws(mi)=(n-kx-ks)*(sols-smin)/smin;
    smins(mi)=smin;
    lams(mi)=lam;
    d1=(q<lam);
    d2=1-d1; 
    xz=[(xs.*(d1*ones(1,length(xs(1,:))))),(xs.*(d2*ones(1,length(xs(1,:))))),xns];
    mmi=inv(xz'*xz);
    bthresh=mmi*(xz'*y);
    e=y-xz*bthresh;
    v=mmi*(e'*e)/(n-kx-ks);
    vd=diag(v);
    ts=bthresh(rur)./sqrt(vd(rur));
    r1(mi)=(ts.*ts)'*(ts<0);
    r2(mi)=ts'*ts;
    t1(mi)=-ts(1);
    t2(mi)=-ts(2);
    mi=mi+1;
end;
[temp,mi]=min(smins);
mhat=ms(mi);
w=ws(mi);
if (1-(m==0))
    mi=m-mmin+1;
end;
lam=lams(mi);
q=qs(:,mi);
d1=(q<lam);
d2=1-d1;
xz=[xs.*(d1*ones(1,length(xs(1,:)))),xs.*(d2*ones(1,length(xs(1,:)))),xns];
mmi=inv(xz'*xz);
bthresh=mmi*(xz'*y);
e=y-xz*bthresh;
ee=e'*e;
v=mmi*(ee)/(n-kx-ks);

vd=diag(v);
v1=vd(1:ks);
v2=vd(ks+1:2*ks);
b1=bthresh(1:ks);
b2=bthresh(ks+1:2*ks);
ts=((b1-b2).*(b1-b2))./(v1+v2);
bs_s=[b1,sqrt(v1),b2,sqrt(v2)];
bs_ns=[bthresh((1+2*ks):(kx+ks)),sqrt(vd((1+2*ks):(kx+ks)))];

